/*
 *	Serial port driver (uses bit-banging)
 *	for 16Cxx series parts.
 *
 * 	IMPORTANT: Compile this file with FULL optimization
 *
 *	Copyright (C)1996 HI-TECH Software.
 *	Freely distributable.
 */
#include	<htc.h>
/*
 *	Tunable parameters
 */

/*	Xtal frequency */
#define	XTAL		4000000

/*	Baud rate	*/
//#define	BRATE		9600
#define	BRATE		9600
//#define	BRATE		38400

/*	Transmit and Receive port bits */
#if ((!defined(TxData)) || (!defined(RxData)))
#define	TxData		GPIO5				/* Map TxData to pin */
#define	RxData		GPIO4				/* Map RxData to pin */
#define	TxDataDir	TRISIO5				/* Map TxData to pin */
#define	RxDataDir	TRISIO4				/* Map RxData to pin */
#endif
#define	INIT_PORT	TxDataDir = 0; RxDataDir = 1	/* set up I/O direction */


/*	Don't change anything else */
#define SCALER			10000000
#define ITIME			4*SCALER/XTAL	/* Instruction cycle time */
#if BRATE > 1200
	#define	DLY			3				/* cycles per null loop */
	#define	TX_OHEAD	13				/* overhead cycles per loop */
#else
	#define	DLY			9				/* cycles per null loop */
	#define TX_OHEAD	14
#endif
//#define	RX_OHEAD		12				/* receiver overhead per loop */
#define	RX_OHEAD		10

#define	DELAY(ohead)	(((SCALER/BRATE)-(ohead*ITIME))/(DLY*ITIME))

void
putch(char c)
{
	unsigned char	bitno;
#if BRATE > 1200
	unsigned char	dly;
#else
	unsigned int	dly;
#endif

	INIT_PORT;
	TxData = 1;			/* start bit */
	bitno = 12;
	do {
		dly = DELAY(TX_OHEAD);	/* wait one bit time */
		do
			/* waiting in delay loop */ ;
		while(--dly);
		if(c & 1)
			TxData = 0;
		if(!(c & 1))
			TxData = 1;
		c = (c >> 1) | 0x80;
	} while(--bitno);
	NOP();
}

char
getch(void)
{
	unsigned char	c, bitno;
#if BRATE > 1200
	unsigned char	dly;
#else
	unsigned int	dly;
#endif

//	for(;;) {
//		dly = DELAY(3);
//		while(--dly && RxData);
		while(RxData)
			continue;		/* wait for start bit */
		dly = DELAY(3)/2;
		do
							/* waiting in delay loop */ ;
		while(--dly);
		if(RxData)
			return 0xff;
			//continue;		/* twas just noise */
		bitno = 8;
		c = 0;
		do {
			dly = DELAY(RX_OHEAD);
			do
							/* waiting in delay loop */ ;
			while(--dly);
			c = (c >> 1) | (RxData << 7);
		} while(--bitno);
		return c;
//	}
}

char
getche(void)
{
	char c;

	putch(c = getch());
	return c;
}
