#ifndef _BOOTLDR_H_
#define _BOOTLDR_H_

/*
 * 	BOOT LOADER v3.11 Copyright (C) 2006 HI-TECH Software
 *	This software is freely distributable and may be used
 *	for any purpose. No warranty of any kind is provided
 *	and all use is entirely at your own risk.
 *
 *	The boot loader is suitable for use with the following
 *	PICmicro controllers:
 *
 *	16F870, 16F871, 16F873, 16F873A, 16F874,
 *	16F874A, 16F876, 16F876A, 16F877, 16F877A
 *	
 *	BOOTLDR.H
 */

/* define the symbol BOUNDED_HEXFILE if you can guarentee
 * that the hex file being downloaded has data records that align on addresses
 * which are a multiple of 4 words and that the length of all data records is
 * also a multiple of 4 words. */
#define BOUNDED_HEXFILE	1

/* Time (seconds) to wait for user input. Must be <= 9 */
#define BOOT_TIMEOUT	9

// What is the program memry size of this chip?
#if defined(__PICCLITE__) && (_ROMSIZE > 0x800)
 #define FLASH_SIZE	0x800	// PICC-Lite has memory limit restrictions to consider
#else
 #define FLASH_SIZE	_ROMSIZE
#endif

// These chips have additional overheads when writing to flash.
// These overheads can be ignored if the input hexfile is conditioned such that
// ALL data records begin on multiples of 4 words (8 bytes) and
// are of a length which is also a multiple of 4 words (8 bytes).
// Hexmate can be used to produce such a file.
// If you can guarentee these conditions on the input file, then define preprocessor
// symbol BOUNDED_HEXFILE.
#if !defined(BOUNDED_HEXFILE) && ( _FLASH_WRITE_SIZE == 4 )
 #define _WRITE_4_WORDS
#endif

// If this device has bank2 available - use it!
#if _GPRBITS_ > 1
 #define BANKX bank2
#else
 #define BANKX near
#endif

#ifndef FLASH_SIZE
 #error Bootloader does not support the selected processor
#endif

/* Address where boot loader will start */
#if VERBOSE == 2
 #define _BOOTLOADER_SIZE 0x180
#elif defined(VERBOSE)
 #define _BOOTLOADER_SIZE 0x180
#else
// #ifdef _WRITE_4_WORDS
//  #define _BOOTLOADER_SIZE 0x130
// #else
  #define _BOOTLOADER_SIZE 0x100
// #endif
#endif

// If this bootloader is being compiled to run on a system that may one day
// be debugged with the ICD2, we need to allow space for that.
#if defined(MPLAB_ICD) && !defined(__PICCLITE__)
 #define BOOT_START (FLASH_SIZE - _BOOTLOADER_SIZE - 0x100)
#else
 #define BOOT_START (FLASH_SIZE - _BOOTLOADER_SIZE)
#endif

#if ((BOOT_START & 0xFF) == 0x00)
 #define _BANK_ALIGNED
#endif

// The bootloader must be aligned on a 4 word boundary
#if ((BOOT_START & 0x03) != 0)
 #error Bootloader must be aligned on a 4 word address boundary (x0h, x4h, x8h, xCh)
#endif

/* Comms setup */
#ifndef BAUD
 #define BAUD 19200
#endif

#ifndef FOSC
 #define FOSC 4000000L
#endif
#define NINE 0     /* Use 9bit communication? FALSE=8bit */

#define DIVIDER ((int)(FOSC/(16UL * BAUD) -1))
#define HIGH_SPEED 1

#if NINE == 1
 #define NINE_BITS 0x40
#else
 #define NINE_BITS 0
#endif

#if HIGH_SPEED == 1
 #define SPEED 0x4
#else
 #define SPEED 0
#endif

#define RX_PIN TRISC7
#define TX_PIN TRISC6
	
/* Serial initialization */
#define INIT_COMMS()\
	RX_PIN = 1;	\
	TX_PIN = 1;		  \
	SPBRG = DIVIDER;     	\
	RCSTA = (NINE_BITS|0x90);	\
	TXSTA = (SPEED|NINE_BITS|0x20)

#ifndef NOP()
 #define NOP()	asm("nop")
#endif

#ifndef RESET()
 #define RESET()	asm("ljmp 0")
#endif

/* expressions used in the code */
#define FLASH EEPGD==1
#define EEPROM EEPGD==0

/* stop & start controls for HyperTerminal */
#define XON	17
#define XOFF	19

#endif
