/* This routine performs a simple checkerboard test on a single cell in RAM.
 * When called, the address being tested is already loaded in FSR, so
 * the test(s) can be preformed via the INDF register.
 * If a cell test detects a failure, call _ram_test_failed
 * with a failure code in the working register.
 */

static volatile unsigned char INDF @ 0;

#ifdef _PIC12
extern void fastcall ram_test_failed(unsigned char);

void fastcall _ram_cell_test(void){
#else
extern void ram_test_failed(unsigned char);

void _ram_cell_test(void){
#endif
	// FSR loaded with test location
	INDF = 0x55;	// test with 55h
	if(INDF!=0x55)
		ram_test_failed(0x55);	// failure code 55h
	INDF = 0xAA;
	if(INDF!=0xAA)	// test with AAh
		ram_test_failed(0xAA);	// failure code AAh
	INDF=0;	// Testing complete
}

