
/* special function registers */

C       equ     0
Z       equ     2
PCL	equ	2

#ifdef	_PIC16
INDF	equ	0
FSR	equ	1
STATUS	equ	4
PCLATH	equ	3
TBLPTRL	equ	0Dh
TBLPTRH	equ	0Eh
WREG	equ	0Ah
BSR	equ	0Fh
PRODL	equ	18h
PRODH	equ	19h
#else
STATUS	equ	3
FSR	equ	4
INDF	equ	0
PCLATH	equ	0Ah
#endif

#ifdef	_PIC12
#define	return	retlw 0
#endif

#ifdef _PIC2
 #define _PAGESIZE 0x200
#elif defined(_PIC14)
 #define _PAGESIZE 0x800
#else
 #define _PAGESIZE 0x2000
#endif

#if _ROMSIZE > _PAGESIZE
 #define _NEEDS_PAGING 1
#endif

; A macro which ensures that the memory bank in which the temp psect
; resides is selected.
; It swaps bank if there is no common area and there is more than
; one bank of memory available.

SELTEMP	macro

// The 17 series is special in that some chips have common memory, but not
// enough to fit all the temp psect.

#if defined _PIC16 && !defined _17C42
	movlr	0h
#endif

#if	(!_COMMON_) && _BANKBITS_

// _PIC12 do nothing

#ifdef	_PIC14
	bcf	STATUS,5
#if	_BANKBITS_ == 2
	bcf	STATUS,6
#endif
#endif	

#endif	

	endm

; A macro to select bank 0

SELBANK0	macro

#if defined _PIC16 && !defined _17C42
	movlr	0h
#endif

#ifdef	_PIC14
	bcf	STATUS,5
#ifdef _USEBANKBITS
#if	_BANKBITS_ == 2
	bcf	STATUS,6
#endif
#else
#if	_GPRBITS_ == 2
	bcf	STATUS,6
#endif
#endif
#endif
	endm

CLRBANK0	macro
#if defined _PIC16 && !defined _17C42
	movlr	0h
#endif

#ifdef	_PIC14
	clrf	STATUS
#endif
	endm


; A macro to select bank 1

SELBANK1	macro

#if defined _PIC16 && !defined _17C42
	movlr	1h
#endif

#ifdef	_PIC14
	bsf	STATUS,5
#ifdef	_USEBANKBITS
#if	_BANKBITS_ == 2
	bcf	STATUS,6
#endif
#else
#if	_GPRBITS_ == 2
	bcf	STATUS,6
#endif
#endif
#endif
	endm

; A macro to select bank 2

SELBANK2	macro

#if defined _PIC16 && !defined _17C42
	movlr	2h
#endif

#ifdef	_PIC14
	bcf	STATUS,5
#if	_BANKBITS_ == 2
	bsf	STATUS,6
#endif
#endif
	endm

; A macro to select bank 3

SELBANK3	macro

#if defined _PIC16 && !defined _17C42
	movlr	3h
#endif

#ifdef	_PIC14
	bsf	STATUS,5
#if	_BANKBITS_ == 2
	bsf	STATUS,6
#endif
#endif
	endm


#ifdef _PROGMEM_

#if	defined(_16C781) || defined(_16C782)
#define _NEEDS_INTS_OFF_
INTSOFF		macro
	bcf	STATUS,0
	btfsc	11,7
	bsf	STATUS,0
	bcf	11,7
	endm

INTSON		macro
	btfsc	STATUS,0
	bsf	11,7
	endm
#endif

#if	defined(_16F99) || (_16F99C)
#define _TEST_CHIP_
#endif

#if	defined(_16F913) || defined(_16F914) || defined(_16F916) || defined(_16F917) ||\
	defined(_16F818) || defined(_16F819) ||\
	defined(_16F72) || defined(_16F73) || defined(_16F74) ||\
       	defined(_16F76) || defined(_16F77) || defined(_16F870) ||\
       	defined(_16F871) || defined(_16F872) || defined(_16F873) ||\
       	defined(_16F873A) || defined(_16F874) || defined(_16F874A) ||\
       	defined(_16F876) || defined(_16F876A) || defined(_16F877) ||\
       	defined(_16F877A) || defined(_16C781) || defined(_16C782) ||\
       	defined(_16F87) || defined(_16F88) ||\
	defined(_16F737) || defined(_16F747) || defined(_16F767) ||\
	defined(_16F685) || defined(_16F687) || defined(_16F689) ||\
       	defined(_16F690) || defined(_16F777) || defined(_16F99C) ||\
       	defined(_16F99) || defined(_16F999C) || defined(_16F999)
#define _PROGMEM_SET_

#define	SEL_PMREGS	SELBANK2
#define SEL_PMCON	SELBANK3

PMADR	set	13
PMADRH	set	15
PMDATA	set	12
PMDATH	set	14
PMCON	set	12

#elif	defined(_16F688)
#define _PROGMEM_SET_

#define	SEL_PMREGS	SELBANK1
#define SEL_PMCON	SELBANK1

PMADR	set	27
PMADRH	set	24
PMDATA	set	26
PMDATH	set	23
PMCON	set	28

#elif	defined(_16C925) || defined(_16C926)
#define _PROGMEM_SET_

#define	SEL_PMREGS	SELBANK3
#define SEL_PMCON	SELBANK2

PMDATA	set	12
PMDATH	set	13
PMADR	set	14
PMADRH	set	15
PMCON	set	12

#endif


#endif



#if	defined(_PROGMEM_) && !defined(_PROGMEM_SET_)
#error Please define PMxxx definitions in SFR.H for reading program memory 
#endif

